@@ -30,7 +30,9 @@
 
 #include <glib/gi18n.h>
 #include <json-glib/json-glib.h>
+#include <inttypes.h>
 #include <libsoup/soup.h>
+#include <math.h>
 #include <string.h>
 
 struct _EphySyncService {
@@ -97,8 +99,8 @@ typedef struct {
   char                *endpoint;
   char                *method;
   char                *request_body;
-  double               modified_since;
-  double               unmodified_since;
+  gint64               modified_since;
+  gint64               unmodified_since;
   SoupSessionCallback  callback;
   gpointer             user_data;
 } StorageRequestAsyncData;
@@ -145,8 +147,8 @@ static StorageRequestAsyncData *
 storage_request_async_data_new (const char          *endpoint,
                                 const char          *method,
                                 const char          *request_body,
-                                double               modified_since,
-                                double               unmodified_since,
+                                gint64               modified_since,
+                                gint64               unmodified_since,
                                 SoupSessionCallback  callback,
                                 gpointer             user_data)
 {
@@ -399,6 +401,11 @@ ephy_sync_service_get_key_bundle (EphySyncService *self,
   g_assert (collection);
 
   crypto_keys = ephy_sync_service_get_secret (self, secrets[CRYPTO_KEYS]);
+  if (!crypto_keys) {
+    g_warning ("Missing crypto-keys secret");
+    return NULL;
+  }
+
   node = json_from_string (crypto_keys, &error);
   g_assert (!error);
   json = json_node_get_object (node);
@@ -539,12 +546,12 @@ ephy_sync_service_send_storage_request (EphySyncService         *self,
     soup_message_headers_append (msg->request_headers, "content-type", content_type);
 
   if (data->modified_since >= 0) {
-    if_modified_since = g_strdup_printf ("%.2lf", data->modified_since);
+    if_modified_since = g_strdup_printf ("%"PRId64, data->modified_since);
     soup_message_headers_append (msg->request_headers, "X-If-Modified-Since", if_modified_since);
   }
 
   if (data->unmodified_since >= 0) {
-    if_unmodified_since = g_strdup_printf ("%.2lf", data->unmodified_since);
+    if_unmodified_since = g_strdup_printf ("%"PRId64, data->unmodified_since);
     soup_message_headers_append (msg->request_headers, "X-If-Unmodified-Since", if_unmodified_since);
   }
 
@@ -1040,8 +1047,8 @@ ephy_sync_service_queue_storage_request (EphySyncService     *self,
                                          const char          *endpoint,
                                          const char          *method,
                                          const char          *request_body,
-                                         double               modified_since,
-                                         double               unmodified_since,
+                                         gint64               modified_since,
+                                         gint64               unmodified_since,
                                          SoupSessionCallback  callback,
                                          gpointer             user_data)
 {
@@ -1109,6 +1116,10 @@ ephy_sync_service_delete_synchronizable (EphySyncService           *self,
   g_assert (ephy_sync_utils_user_is_signed_in ());
 
   collection = ephy_synchronizable_manager_get_collection_name (manager);
+  bundle = ephy_sync_service_get_key_bundle (self, collection);
+  if (!bundle)
+    return;
+
   id = ephy_synchronizable_get_id (synchronizable);
   /* Firefox uses UUIDs with curly braces as IDs for saved passwords records.
    * Curly braces are unsafe characters in URLs so they must be encoded.
@@ -1122,7 +1133,6 @@ ephy_sync_service_delete_synchronizable (EphySyncService           *self,
   json_object_set_string_member (object, "id", id);
   json_object_set_boolean_member (object, "deleted", TRUE);
   record = json_to_string (node, FALSE);
-  bundle = ephy_sync_service_get_key_bundle (self, collection);
   payload = ephy_sync_crypto_encrypt_record (record, bundle);
   json_object_remove_member (object, "deleted");
   json_object_set_string_member (object, "payload", payload);
@@ -1170,6 +1180,9 @@ download_synchronizable_cb (SoupSession *session,
   type = ephy_synchronizable_manager_get_synchronizable_type (data->manager);
   collection = ephy_synchronizable_manager_get_collection_name (data->manager);
   bundle = ephy_sync_service_get_key_bundle (data->service, collection);
+  if (!bundle)
+    goto out;
+
   synchronizable = EPHY_SYNCHRONIZABLE (ephy_synchronizable_from_bso (node, type, bundle, &is_deleted));
   if (!synchronizable) {
     g_warning ("Failed to create synchronizable object from BSO");
@@ -1236,7 +1249,7 @@ upload_synchronizable_cb (SoupSession *session,
                           gpointer     user_data)
 {
   SyncAsyncData *data = (SyncAsyncData *)user_data;
-  double time_modified;
+  gint64 time_modified;
 
   /* Code 412 means that there is a more recent version on the server.
    * Download it.
@@ -1246,7 +1259,7 @@ upload_synchronizable_cb (SoupSession *session,
     ephy_sync_service_download_synchronizable (data->service, data->manager, data->synchronizable);
   } else if (msg->status_code == 200) {
     LOG ("Successfully uploaded to server");
-    time_modified = g_ascii_strtod (msg->response_body->data, NULL);
+    time_modified = ceil (g_ascii_strtod (msg->response_body->data, NULL));
     ephy_synchronizable_set_server_time_modified (data->synchronizable, time_modified);
     ephy_synchronizable_manager_save (data->manager, data->synchronizable);
   } else {
@@ -1271,7 +1284,7 @@ ephy_sync_service_upload_synchronizable (EphySyncService           *self,
   char *id_safe;
   const char *collection;
   const char *id;
-  double time_modified;
+  gint64 time_modified;
 
   g_assert (EPHY_IS_SYNC_SERVICE (self));
   g_assert (EPHY_IS_SYNCHRONIZABLE_MANAGER (manager));
@@ -1280,6 +1293,9 @@ ephy_sync_service_upload_synchronizable (EphySyncService           *self,
 
   collection = ephy_synchronizable_manager_get_collection_name (manager);
   bundle = ephy_sync_service_get_key_bundle (self, collection);
+  if (!bundle)
+    return;
+
   bso = ephy_synchronizable_to_bso (synchronizable, bundle);
   id = ephy_synchronizable_get_id (synchronizable);
   /* Firefox uses UUIDs with curly braces as IDs for saved passwords records.
@@ -1318,9 +1334,12 @@ ephy_sync_service_split_into_batches (EphySyncService           *self,
   g_assert (EPHY_IS_SYNCHRONIZABLE_MANAGER (manager));
   g_assert (synchronizables);
 
-  batches = g_ptr_array_new_with_free_func (g_free);
   collection = ephy_synchronizable_manager_get_collection_name (manager);
   bundle = ephy_sync_service_get_key_bundle (self, collection);
+  if (!bundle)
+    return NULL;
+
+  batches = g_ptr_array_new_with_free_func (g_free);
 
   for (guint i = start; i < end; i += EPHY_SYNC_BATCH_SIZE) {
     JsonNode *node = json_node_new (JSON_NODE_ARRAY);
@@ -1500,7 +1519,7 @@ sync_collection_cb (SoupSession *session,
 {
   SyncCollectionAsyncData *data = (SyncCollectionAsyncData *)user_data;
   EphySynchronizable *remote;
-  SyncCryptoKeyBundle *bundle;
+  SyncCryptoKeyBundle *bundle = NULL;
   JsonNode *node = NULL;
   JsonArray *array = NULL;
   GError *error = NULL;
@@ -1528,6 +1547,9 @@ sync_collection_cb (SoupSession *session,
 
   type = ephy_synchronizable_manager_get_synchronizable_type (data->manager);
   bundle = ephy_sync_service_get_key_bundle (data->service, collection);
+  if (!bundle)
+    goto out_error;
+
   for (guint i = 0; i < json_array_get_length (array); i++) {
     remote = EPHY_SYNCHRONIZABLE (ephy_synchronizable_from_bso (json_array_get_element (array, i),
                                                                 type, bundle, &is_deleted));
@@ -1557,6 +1579,8 @@ out_error:
     g_signal_emit (data->service, signals[SYNC_FINISHED], 0);
   sync_collection_async_data_free (data);
 out_no_error:
+  if (bundle)
+    ephy_sync_crypto_key_bundle_free (bundle);
   if (node)
     json_node_unref (node);
   if (error)
@@ -1583,7 +1607,7 @@ ephy_sync_service_sync_collection (EphySyncService           *self,
   if (is_initial) {
     endpoint = g_strdup_printf ("storage/%s?full=true", collection);
   } else {
-    endpoint = g_strdup_printf ("storage/%s?newer=%.2lf&full=true", collection,
+    endpoint = g_strdup_printf ("storage/%s?newer=%"PRId64"&full=true", collection,
                                 ephy_synchronizable_manager_get_sync_time (manager));
   }
 
@@ -1858,6 +1882,10 @@ ephy_sync_service_upload_client_record (EphySyncService *self)
 
   g_assert (EPHY_IS_SYNC_SERVICE (self));
 
+  bundle = ephy_sync_service_get_key_bundle (self, "clients");
+  if (!bundle)
+    return;
+
   /* Make device ID and name. */
   device_bso_id = ephy_sync_utils_get_device_bso_id ();
   device_id = ephy_sync_utils_get_device_id ();
@@ -1865,7 +1893,6 @@ ephy_sync_service_upload_client_record (EphySyncService *self)
 
   /* Make BSO as string. */
   record = ephy_sync_utils_make_client_record (device_bso_id, device_id, device_name);
-  bundle = ephy_sync_service_get_key_bundle (self, "clients");
   encrypted = ephy_sync_crypto_encrypt_record (record, bundle);
 
   bso = json_object_new ();
@@ -2770,9 +2797,10 @@ void
 ephy_sync_service_start_sync (EphySyncService *self)
 {
   g_assert (EPHY_IS_SYNC_SERVICE (self));
-  g_assert (ephy_sync_utils_user_is_signed_in ());
   g_assert (self->sync_periodically);
 
-  ephy_sync_service_sync_internal (self);
-  ephy_sync_service_schedule_periodical_sync (self);
+  if (ephy_sync_utils_user_is_signed_in ()) {
+    ephy_sync_service_sync_internal (self);
+    ephy_sync_service_schedule_periodical_sync (self);
+  }
 }
